@@ -1,37 +1,7 @@
-///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004, Industrial Light & Magic, a division of Lucas
-// Digital Ltd. LLC
-// 
-// All rights reserved.
-// 
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-// *       Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-// *       Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-// *       Neither the name of Industrial Light & Magic nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission. 
-// 
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// SPDX-License-Identifier: BSD-3-Clause
+// Copyright (c) Contributors to the OpenEXR Project.
 //
-///////////////////////////////////////////////////////////////////////////
-
 
 
 #ifndef INCLUDED_IMF_HEADER_H
@@ -43,28 +13,30 @@
 //
 //-----------------------------------------------------------------------------
 
-#include <ImfLineOrder.h>
-#include <ImfCompression.h>
-#include <ImfName.h>
-#include <ImfTileDescription.h>
-#include <ImfInt64.h>
+#include "ImfForward.h"
+
+#include "ImfLineOrder.h"
+#include "ImfCompression.h"
+#include "ImfName.h"
+#include "ImfTileDescription.h"
 #include "ImathVec.h"
 #include "ImathBox.h"
 #include "IexBaseExc.h"
+
+#include "ImfAttribute.h"
+
 #include <map>
 #include <iosfwd>
-
-namespace Imf {
+#include <string>
+#include <cstdint>
 
 
-class Attribute;
-class ChannelList;
-class IStream;
-class OStream;
-class PreviewImage;
+OPENEXR_IMF_INTERNAL_NAMESPACE_HEADER_ENTER
+
+using std::string;
 
 
-class Header
+class IMF_EXPORT_TYPE Header
 {
   public:
     
@@ -73,10 +45,11 @@ class Header
     // are both set to Box2i (V2i (0, 0), V2i (width-1, height-1).
     //----------------------------------------------------------------
 
+    IMF_EXPORT
     Header (int width = 64,
 	    int height = 64,
 	    float pixelAspectRatio = 1,
-	    const Imath::V2f &screenWindowCenter = Imath::V2f (0, 0),
+	    const IMATH_NAMESPACE::V2f &screenWindowCenter = IMATH_NAMESPACE::V2f (0, 0),
 	    float screenWindowWidth = 1,
 	    LineOrder lineOrder = INCREASING_Y,
 	    Compression = ZIP_COMPRESSION);
@@ -87,11 +60,12 @@ class Header
     // window is set to Box2i (V2i (0, 0), V2i (width-1, height-1).
     //--------------------------------------------------------------------
 
+    IMF_EXPORT
     Header (int width,
 	    int height,
-	    const Imath::Box2i &dataWindow,
+	    const IMATH_NAMESPACE::Box2i &dataWindow,
 	    float pixelAspectRatio = 1,
-	    const Imath::V2f &screenWindowCenter = Imath::V2f (0, 0),
+	    const IMATH_NAMESPACE::V2f &screenWindowCenter = IMATH_NAMESPACE::V2f (0, 0),
 	    float screenWindowWidth = 1,
 	    LineOrder lineOrder = INCREASING_Y,
 	    Compression = ZIP_COMPRESSION);
@@ -102,10 +76,11 @@ class Header
     // both specified explicitly.
     //----------------------------------------------------------
 
-    Header (const Imath::Box2i &displayWindow,
-	    const Imath::Box2i &dataWindow,
+    IMF_EXPORT
+    Header (const IMATH_NAMESPACE::Box2i &displayWindow,
+	    const IMATH_NAMESPACE::Box2i &dataWindow,
 	    float pixelAspectRatio = 1,
-	    const Imath::V2f &screenWindowCenter = Imath::V2f (0, 0),
+	    const IMATH_NAMESPACE::V2f &screenWindowCenter = IMATH_NAMESPACE::V2f (0, 0),
 	    float screenWindowWidth = 1,
 	    LineOrder lineOrder = INCREASING_Y,
 	    Compression = ZIP_COMPRESSION);
@@ -115,13 +90,16 @@ class Header
     // Copy constructor
     //-----------------
 
-    Header (const Header &other);
-
+    IMF_EXPORT
+    Header (const Header& other);
+    IMF_EXPORT
+    Header (Header&& other);
 
     //-----------
     // Destructor
     //-----------
 
+    IMF_EXPORT
     ~Header ();
 
 
@@ -129,8 +107,10 @@ class Header
     // Assignment
     //-----------
 
-    Header &			operator = (const Header &other);
-
+    IMF_EXPORT
+    Header& operator= (const Header& other);
+    IMF_EXPORT
+    Header& operator= (Header&& other);
 
     //---------------------------------------------------------------
     // Add an attribute:
@@ -145,27 +125,50 @@ class Header
     //			is copied into this attribute.
     //
     //			If an attribute with name n exists, and its
-    //			type is different from attr, an Iex::TypeExc
+    //			type is different from attr, an IEX_NAMESPACE::TypeExc
     //			is thrown.
     //
     //---------------------------------------------------------------
 
+    IMF_EXPORT
     void			insert (const char name[],
 				        const Attribute &attribute);
 
+    IMF_EXPORT
+    void			insert (const std::string &name,
+				        const Attribute &attribute);
+
+    //---------------------------------------------------------------
+    // Remove an attribute:
+    //
+    // remove(n)       If an attribute with name n exists, then it
+    //                 is removed from the map of present attributes.
+    //
+    //                 If no attribute with name n exists, then this
+    //                 functions becomes a 'no-op'
+    //
+    //---------------------------------------------------------------
+
+    IMF_EXPORT
+    void                        erase (const char name[]);
+    IMF_EXPORT
+    void                        erase (const std::string &name);
+
+    
+    
     //------------------------------------------------------------------
     // Access to existing attributes:
     //
     // [n]			Returns a reference to the attribute
     //				with name n.  If no attribute with
-    //				name n exists, an Iex::ArgExc is thrown.
+    //				name n exists, an IEX_NAMESPACE::ArgExc is thrown.
     //
     // typedAttribute<T>(n)	Returns a reference to the attribute
     //				with name n and type T.  If no attribute
-    //				with name n exists, an Iex::ArgExc is
+    //				with name n exists, an IEX_NAMESPACE::ArgExc is
     //				thrown.  If an attribute with name n
     //				exists, but its type is not T, an
-    //				Iex::TypeExc is thrown.
+    //				IEX_NAMESPACE::TypeExc is thrown.
     //
     // findTypedAttribute<T>(n)	Returns a pointer to the attribute with
     //				name n and type T, or 0 if no attribute
@@ -173,15 +176,28 @@ class Header
     //
     //------------------------------------------------------------------
 
+    IMF_EXPORT
     Attribute &			operator [] (const char name[]);
+    IMF_EXPORT
     const Attribute &		operator [] (const char name[]) const;
 
+    IMF_EXPORT
+    Attribute &			operator [] (const std::string &name);
+    IMF_EXPORT
+    const Attribute &		operator [] (const std::string &name) const;
+
     template <class T> T&	typedAttribute (const char name[]);
     template <class T> const T&	typedAttribute (const char name[]) const;
 
+    template <class T> T&	typedAttribute (const std::string &name);
+    template <class T> const T&	typedAttribute (const std::string &name) const;
+
     template <class T> T*	findTypedAttribute (const char name[]);
     template <class T> const T*	findTypedAttribute (const char name[]) const;
 
+    template <class T> T*	findTypedAttribute (const std::string &name);
+    template <class T> const T*	findTypedAttribute (const std::string &name)
+								       const;
 
     //---------------------------------------------
     // Iterator-style access to existing attributes
@@ -192,42 +208,156 @@ class Header
     class Iterator;
     class ConstIterator;
 
+    IMF_EXPORT
     Iterator			begin ();
+    IMF_EXPORT
     ConstIterator		begin () const;
+
+    IMF_EXPORT
     Iterator			end ();
+    IMF_EXPORT
     ConstIterator		end () const;
+
+    IMF_EXPORT
     Iterator			find (const char name[]);
+    IMF_EXPORT
     ConstIterator		find (const char name[]) const;
 
+    IMF_EXPORT
+    Iterator			find (const std::string &name);
+    IMF_EXPORT
+    ConstIterator		find (const std::string &name) const;
+
 
     //--------------------------------
     // Access to predefined attributes
     //--------------------------------
 
-    Imath::Box2i &		displayWindow ();
-    const Imath::Box2i &	displayWindow () const;
+    IMF_EXPORT
+    IMATH_NAMESPACE::Box2i &		displayWindow ();
+    IMF_EXPORT
+    const IMATH_NAMESPACE::Box2i &	displayWindow () const;
 
-    Imath::Box2i &		dataWindow ();
-    const Imath::Box2i &	dataWindow () const;
+    IMF_EXPORT
+    IMATH_NAMESPACE::Box2i &		dataWindow ();
+    IMF_EXPORT
+    const IMATH_NAMESPACE::Box2i &	dataWindow () const;
 
+    IMF_EXPORT
     float &			pixelAspectRatio ();
+    IMF_EXPORT
     const float &		pixelAspectRatio () const;
 
-    Imath::V2f &		screenWindowCenter ();
-    const Imath::V2f &		screenWindowCenter () const;
+    IMF_EXPORT
+    IMATH_NAMESPACE::V2f &		screenWindowCenter ();
+    IMF_EXPORT
+    const IMATH_NAMESPACE::V2f &		screenWindowCenter () const;
 
+    IMF_EXPORT
     float &			screenWindowWidth ();
+    IMF_EXPORT
     const float &		screenWindowWidth () const;
 
+    IMF_EXPORT
     ChannelList &		channels ();
+    IMF_EXPORT
     const ChannelList &		channels () const;
 
+    IMF_EXPORT
     LineOrder &			lineOrder ();
+    IMF_EXPORT
     const LineOrder &		lineOrder () const;
 
+    IMF_EXPORT
     Compression &		compression ();
+    IMF_EXPORT
     const Compression &		compression () const;
 
+    //-----------------------------------------------------
+    // The header object allows one to store a compression level to be
+    // used when writing a file.
+    //
+    // NB: These are NOT attributes, and will not be written to the
+    // file, but are instead ephemeral settings to be used for this
+    // instance of the header object.
+    //
+    // -----------------------------------------------------
+    IMF_EXPORT
+    void resetDefaultCompressionLevels ();
+    IMF_EXPORT
+    int& zipCompressionLevel ();
+    IMF_EXPORT
+    int zipCompressionLevel () const;
+    IMF_EXPORT
+    float& dwaCompressionLevel ();
+    IMF_EXPORT
+    float dwaCompressionLevel () const;
+
+    //-----------------------------------------------------
+    // Access to required attributes for multipart files
+    // They are optional to non-multipart files and mandatory
+    // for multipart files.
+    //-----------------------------------------------------
+    IMF_EXPORT
+    void                        setName (const string& name);
+
+    IMF_EXPORT
+    string&                     name();
+    IMF_EXPORT
+    const string&               name() const;
+
+    IMF_EXPORT
+    bool                        hasName() const;
+
+    IMF_EXPORT
+    void                        setType (const string& Type);
+
+    IMF_EXPORT
+    string&                     type();
+    IMF_EXPORT
+    const string&               type() const;
+
+    IMF_EXPORT
+    bool                        hasType() const;
+
+    IMF_EXPORT
+    void                        setVersion (const int version);
+
+    IMF_EXPORT
+    int&                        version();
+    IMF_EXPORT
+    const int&                  version() const;
+
+    IMF_EXPORT
+    bool                        hasVersion() const;
+
+    //
+    // the chunkCount attribute is set automatically when a file is written.
+    // There is no need to set it manually
+    //
+    IMF_EXPORT
+    void                        setChunkCount(int chunks);
+    IMF_EXPORT
+    bool                        hasChunkCount() const;
+    IMF_EXPORT
+    const int &                 chunkCount() const;
+    IMF_EXPORT
+    int &                       chunkCount();
+
+    
+    //
+    // for multipart files, return whether the file has a view string attribute
+    // (for the deprecated single part multiview format EXR, see ImfMultiView.h)
+    //
+    IMF_EXPORT
+    void                       setView(const string & view);
+    IMF_EXPORT
+    bool                       hasView() const;
+    IMF_EXPORT
+    string &                   view();
+    IMF_EXPORT
+    const string &             view() const;
+    
 
     //----------------------------------------------------------------------
     // Tile Description:
@@ -250,11 +380,15 @@ class Header
     //
     //----------------------------------------------------------------------
 
+    IMF_EXPORT
     void			setTileDescription (const TileDescription & td);
 
+    IMF_EXPORT
     TileDescription &		tileDescription ();
+    IMF_EXPORT
     const TileDescription &	tileDescription () const;
 
+    IMF_EXPORT
     bool			hasTileDescription() const;
 
 
@@ -279,11 +413,15 @@ class Header
     //
     //----------------------------------------------------------------------
 
+    IMF_EXPORT
     void			setPreviewImage (const PreviewImage &p);
 
+    IMF_EXPORT
     PreviewImage &		previewImage ();
+    IMF_EXPORT
     const PreviewImage &	previewImage () const;
 
+    IMF_EXPORT
     bool			hasPreviewImage () const;
 
 
@@ -296,7 +434,9 @@ class Header
     // header
     //-------------------------------------------------------------
 
-    void			sanityCheck (bool isTiled = false) const;
+    IMF_EXPORT
+    void			sanityCheck (bool isTiled = false,
+        			             bool isMultipartFile = false) const;
 
 
     //----------------------------------------------------------------
@@ -315,9 +455,17 @@ class Header
     // a damaged image file.
     //----------------------------------------------------------------
 
+    IMF_EXPORT
     static void			setMaxImageSize (int maxWidth, int maxHeight);
+    IMF_EXPORT
     static void			setMaxTileSize (int maxWidth, int maxHeight);
 
+    //
+    // Check if the header reads nothing.
+    //
+    IMF_EXPORT
+    bool                        readsNothing();
+
 
     //------------------------------------------------------------------
     // Input and output:
@@ -330,14 +478,19 @@ class Header
     //------------------------------------------------------------------
 
 
-    Int64			writeTo (OStream &os,
+    IMF_EXPORT
+    uint64_t			writeTo (OPENEXR_IMF_INTERNAL_NAMESPACE::OStream &os,
 					 bool isTiled = false) const;
 
-    void			readFrom (IStream &is, int &version);
+    IMF_EXPORT
+    void			readFrom (OPENEXR_IMF_INTERNAL_NAMESPACE::IStream &is,
+        			          int &version);
+    
 
-  private:
+private:
+    AttributeMap _map;
 
-    AttributeMap		_map;
+    bool _readsNothing;
 };
 
 
@@ -345,17 +498,23 @@ class Header
 // Iterators
 //----------
 
-class Header::Iterator
+class IMF_EXPORT_TYPE Header::Iterator
 {
   public:
 
+    IMF_EXPORT
     Iterator ();
+    IMF_EXPORT
     Iterator (const Header::AttributeMap::iterator &i);
 
+    IMF_EXPORT
     Iterator &			operator ++ ();
+    IMF_EXPORT
     Iterator 			operator ++ (int);
 
+    IMF_EXPORT
     const char *		name () const;
+    IMF_EXPORT
     Attribute &			attribute () const;
 
   private:
@@ -366,18 +525,25 @@ class Header::Iterator
 };
 
 
-class Header::ConstIterator
+class IMF_EXPORT_TYPE Header::ConstIterator
 {
   public:
 
+    IMF_EXPORT
     ConstIterator ();
+    IMF_EXPORT
     ConstIterator (const Header::AttributeMap::const_iterator &i);
+    IMF_EXPORT
     ConstIterator (const Header::Iterator &other);
 
+    IMF_EXPORT
     ConstIterator &		operator ++ ();
+    IMF_EXPORT
     ConstIterator 		operator ++ (int);
 
+    IMF_EXPORT
     const char *		name () const;
+    IMF_EXPORT
     const Attribute &		attribute () const;
 
   private:
@@ -394,7 +560,7 @@ class Header::ConstIterator
 //
 // In a multithreaded program, staticInitialize() must be called once
 // during startup, before the program accesses any other functions or
-// classes in the IlmImf library.  Calling staticInitialize() in this
+// classes in the OpenEXR library.  Calling staticInitialize() in this
 // way avoids races during initialization of the library's global
 // variables.
 //
@@ -403,7 +569,7 @@ class Header::ConstIterator
 //
 //------------------------------------------------------------------------
 
-void staticInitialize ();
+IMF_EXPORT void staticInitialize ();
 
 
 //-----------------
@@ -531,10 +697,10 @@ T &
 Header::typedAttribute (const char name[])
 {
     Attribute *attr = &(*this)[name];
-    T *tattr = dynamic_cast <T*> (attr);
+    T *tattr = dynamic_cast <T *> (attr);
 
     if (tattr == 0)
-	throw Iex::TypeExc ("Unexpected attribute type.");
+	throw IEX_NAMESPACE::TypeExc ("Unexpected attribute type.");
 
     return *tattr;
 }
@@ -545,21 +711,37 @@ const T &
 Header::typedAttribute (const char name[]) const
 {
     const Attribute *attr = &(*this)[name];
-    const T *tattr = dynamic_cast <const T*> (attr);
+    const T *tattr = dynamic_cast <const T *> (attr);
 
     if (tattr == 0)
-	throw Iex::TypeExc ("Unexpected attribute type.");
+	throw IEX_NAMESPACE::TypeExc ("Unexpected attribute type.");
 
     return *tattr;
 }
 
 
+template <class T>
+T &
+Header::typedAttribute (const std::string &name)
+{
+    return typedAttribute<T> (name.c_str());
+}
+
+
+template <class T>
+const T &
+Header::typedAttribute (const std::string &name) const
+{
+    return typedAttribute<T> (name.c_str());
+}
+
+
 template <class T>
 T *
 Header::findTypedAttribute (const char name[])
 {
     AttributeMap::iterator i = _map.find (name);
-    return (i == _map.end())? 0: dynamic_cast <T*> (i->second);
+    return (i == _map.end())? 0: dynamic_cast <T *> (i->second);
 }
 
 
@@ -568,10 +750,26 @@ const T *
 Header::findTypedAttribute (const char name[]) const
 {
     AttributeMap::const_iterator i = _map.find (name);
-    return (i == _map.end())? 0: dynamic_cast <const T*> (i->second);
+    return (i == _map.end())? 0: dynamic_cast <const T *> (i->second);
 }
 
 
-} // namespace Imf
+template <class T>
+T *
+Header::findTypedAttribute (const std::string &name)
+{
+    return findTypedAttribute<T> (name.c_str());
+}
+
+
+template <class T>
+const T *
+Header::findTypedAttribute (const std::string &name) const
+{
+    return findTypedAttribute<T> (name.c_str());
+}
+
+
+OPENEXR_IMF_INTERNAL_NAMESPACE_HEADER_EXIT
 
 #endif
